/***************************************************************************
 *   Copyright (C) 2006 by Michael Andreen                                 *
 *   andreen@student.chalmers.se                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

#include "modelidentifier.h"
#include "genealgorithms.h"

#include <doublefann.h>

using namespace std;

ModelIdentifier::ModelIdentifier(std::string ann){
	_ann = fann_create_from_file(ann.c_str());
	if(!_ann){
		throw invalid_argument("Could not create network");
	}
}

ModelIdentifier::~ModelIdentifier(){
	fann_destroy(_ann);
}

std::map<ModelIdentifier::Model,double> ModelIdentifier::identify(const GeneOrder& go){
	int pos = 0;
	int neg = 0;
	for (GeneOrder::iterator g = go.begin(); g != go.end(); ++g){
		if (*g >= 0)
			++pos;
		else
			++neg;
	}
	double length = go.size();
	vector<double> input(8);
	input[0] = pos/length;
	input[1] = neg/length;

	pair<int,int> seqs = longestSequences(go);

	input[2] = seqs.first/length;
	input[3] = seqs.second/length;

	double cycles = countCycles(go);
	input[4] = cycles/length;

	vector<Component> comps = findComponents(go);

	pos = 0;
	neg = 0;
	int un = 0;
	for (vector<Component>::iterator c = comps.begin(); c != comps.end(); ++c){
		if (c->sign > 0)
			++pos;
		else if (c->sign < 0)
			++neg;
		else
			++un;
	}

	input[5] = un/cycles;
	input[6] = pos/cycles;
	input[7] = neg/cycles;

	double *output = fann_run(_ann,&input[0]);

	map<Model,double> scores;
	scores[X] = output[0];
	scores[Zipper] = output[1];
	scores[Whirl] = output[2];
	scores[FatX] = output[3];
	scores[Cloud] = output[4];
	return scores;
}
