/***************************************************************************
 *   Copyright (C) 2006 by Michael Andreen                                 *
 *   andreen@student.chalmers.se                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

#ifndef __SORTACTION_H__
#define __SORTACTION_H__

#include <tr1/memory>
class GeneOrder;
/**
 * Abstraction of a sort action, all child actions has to be immutable.
 *
 * \author Michael Andreen
 */
class SortAction{
	public:
		typedef std::tr1::shared_ptr<SortAction> ActionPointer;

		/**
		 * Creates a new sort action, given a specified action.
		 * SortAction promises to remove the given action.
		 */
		SortAction(SortAction* sa): _action(sa){
		}
		
		virtual ~SortAction(){};

		/**
		 * Applies the action on the GeneOrder and returning it.
		 */
		virtual GeneOrder& operator()(GeneOrder& go) const{
			return (*_action)(go);
		}

		/**
		 * Compares sort actions.
		 */
		virtual bool operator==(const SortAction& sa) const{
			return (*_action) == (sa._action.get() == 0 ? sa : *sa._action);
		}
	private:
		ActionPointer _action;

};

#endif

